# -*- coding: utf-8 -*-
"""
Day template views
"""
import datetime

from django.conf import settings
from django.views import generic
from django.core.urlresolvers import reverse
from django.contrib.auth.models import User
from django.shortcuts import get_object_or_404

from braces.views import LoginRequiredMixin

from datebook.models import DayEntry, DayModel
from datebook.views.day import DayEntryFormEditView
from datebook.forms.daymodel import DayToDayModelForm
from datebook.mixins import AuthorKwargsMixin, DatebookCalendarMixin, OwnerOrPermissionRequiredMixin


class DayModelListView(AuthorKwargsMixin, OwnerOrPermissionRequiredMixin, generic.ListView):
    """
    Author's day models index
    """
    model = DayModel
    template_name = "datebook/daymodel/index.html"
    permission_required = 'datebook.change_daymodel'
    paginate_by = None
    
    def get_queryset(self):
        return self.model.objects.filter(author=self.author)


class DayEntryToDayModelFormView(DatebookCalendarMixin, OwnerOrPermissionRequiredMixin, generic.CreateView):
    """
    Form view to create a DayModel object from a DayEntry object
    """
    context_object_name = "dayentry"
    template_name = "datebook/day/form.html"
    form_class = DayToDayModelForm
    permission_required = 'datebook.add_daymodel'
    raise_exception = True

    def get_form(self, form_class):
        """
        Add required args to form instance
        """
        return form_class(self.author, **self.get_form_kwargs())
    
    def get_form_kwargs(self):
        """
        Add post form url
        """
        kwargs = super(DayEntryToDayModelFormView, self).get_form_kwargs()
        
        day_date = self.datebook.period.replace(day=self.day)
        time_start = datetime.time(9, 0)
        time_stop = datetime.time(18, 0)
        
        kwargs.update({
            'form_action': '.',
            'dayentry': self.dayentry,
            'day_date': day_date,
            'initial': {
                'start' : self.dayentry.start,
                'stop' : self.dayentry.stop,
                'pause' : self.dayentry.pause,
                'overtime' : self.dayentry.overtime,
                'content' : self.dayentry.content,
            },
        })
        return kwargs


    def get_success_url(self):
        """
        If 'submit_and_next' button has been used, redirect to the next day form, 
        else redirect to the month view
        """
        return reverse('datebook:day-models', kwargs={
            'author': self.author,
        })
    
    def get_dayentry(self):
        """
        Add required args to form instance
        """
        return self.datebook.dayentry_set.get(activity_date__day=self.day)
        
    def get(self, request, *args, **kwargs):
        self.datebook = self.get_datebook({'period__year': self.year, 'period__month': self.month})
        self.author = self.datebook.author
        self.dayentry = self.get_dayentry()
        return super(DayEntryToDayModelFormView, self).get(request, *args, **kwargs)
    
    def post(self, request, *args, **kwargs):
        self.datebook = self.get_datebook({'period__year': self.year, 'period__month': self.month})
        self.author = self.datebook.author
        self.dayentry = self.get_dayentry()
        return super(DayEntryToDayModelFormView, self).post(request, *args, **kwargs)



class DayModelFormEditView(AuthorKwargsMixin, OwnerOrPermissionRequiredMixin, generic.UpdateView):
    """
    DayModel form edit view
    """
    model = DayModel
    context_object_name = "daymodel"
    template_name = "datebook/day/form.html"
    form_class = DayToDayModelForm
    permission_required = 'datebook.change_daymodel'
    raise_exception = True

    def get_form(self, form_class):
        """
        Add required args to form instance
        """
        return form_class(self.author, **self.get_form_kwargs())
    
    def get_object(self, queryset=None):
        return get_object_or_404(self.model, author=self.author, pk=self.kwargs['pk'])
    
    def get_form_kwargs(self):
        """
        Add post form url
        """
        kwargs = super(DayModelFormEditView, self).get_form_kwargs()
        
        kwargs.update({
            'form_action': '.',
            'day_date': self.object.start.date(),
        })
        return kwargs


    def get_success_url(self):
        """
        If 'submit_and_next' button has been used, redirect to the next day form, 
        else redirect to the month view
        """
        return reverse('datebook:day-models', kwargs={
            'author': self.author,
        })




"""
Crispy forms layouts
"""
from django import forms
from django.utils.translation import ugettext as _

from crispy_forms.helper import FormHelper
from crispy_forms_foundation.layout import Layout, Row, Column, HTML, Div, Field, ButtonHolder, ButtonHolderPanel, ButtonGroup, Panel, Submit

def SimpleRowColumn(field, *args, **kwargs):
    """
    Shortcut for simple row with only a full column
    """
    if isinstance(field, basestring):
        field = Field(field, *args, **kwargs)
    return Row(
        Column(field),
    )

def day_helper(form_tag=True, form_action='.', **kwargs):
    """
    DayEntry's form layout helper
    """
    helper = FormHelper()
    helper.form_action = form_action
    helper.attrs = {'data_abide': ''}
    helper.form_tag = form_tag
    
    # Tuple containing next day date and resolved url to its form
    next_day = kwargs.pop('next_day', None)
    # Mode to ensure compatibility with DayToDayModelForm (just a save button and no vacation)
    day_to_model_mode = kwargs.pop('day_to_model_mode', False)
    # Resolved url to the 'DayToDayModelForm' form
    day_to_model_url = kwargs.pop('day_to_model_url', None)
    # Resolved url to the day removing form
    remove_url = kwargs.pop('remove_url', None)
    
    # Menu elements (buttons and links)
    menu_elements = []
    
    # Build button groups
    buttons = [Submit('submit', _('Save'))]
    if not day_to_model_mode and next_day:
        buttons.append(Submit('submit_and_next', _('Save and continue to next day')))
    groups = []
    if remove_url:
        groups.append(ButtonGroup(
            Div(
                HTML('<a class="button tiny alert" href="{0}">'.format(remove_url)),
                HTML(_('Remove this entry')),
                HTML('</a>'),
            ),
            css_class='left'
        ))
    groups.append(ButtonGroup(
        *buttons,
        css_class='right'
    ))
    menu_elements.append(
        Div(
            *groups,
            css_class='clearfix'
        )
    )
        
    # For some links under the form's buttons
    links = []
    if not day_to_model_mode:
        if next_day:
            links.append( 
                Div(
                    HTML('<a class="button tiny secondary" href="{0}">'.format(next_day[1])),
                    HTML(_('Pass and continue to next day')),
                    HTML('</a>'),
                )
            )
        if day_to_model_url:
            links.append( 
                Div(
                    HTML('<a class="button tiny secondary" href="{0}">'.format(day_to_model_url)),
                    HTML(_('Use it as a day model')),
                    HTML('</a>'),
                )
            )
    if links:
        menu_elements.append(
            Div(
                ButtonGroup(
                    *links,
                    css_class='right'
                ),
                css_class='clearfix'
            )
        )
    
    # Menu where belong the buttons and links
    menu = Panel(
        *menu_elements,
        css_class='text-right'
    )
    
    # Build the full layout
    helper.layout = Layout(
        Row(
            Column('vacation', css_class='small-6 medium-4 medium-offset-8 text-right'),
        ),
        Row(
            Column('start_datetime', css_class='small-6 medium-4'),
            Column('pause', css_class='small-6 medium-2'),
            Column('stop_datetime', css_class='small-6 medium-4'),
            Column('overtime', css_class='small-6 medium-2'),
            css_class='opacited'
        ),
        Row(
            Column('content', css_class='small-12'),
            css_class='opacited'
        ),
        menu,
    )
    # Replace 'vacation' field with 'title' field for DayToDayModelForm form
    if day_to_model_mode:
        helper.layout.pop(0)
        helper.layout.insert(0,
            Row(
                Column('title'),
            ),
        )

    return helper


def month_helper(form_tag=True):
    """
    Datebook's month form layout helper
    """
    helper = FormHelper()
    helper.form_action = '.'
    helper.attrs = {'data_abide': ''}
    helper.form_tag = form_tag
    
    # Build the full layout
    helper.layout = Layout(
        Row(
            Column(
                Field('owner'),
                css_class='small-12 medium-7 hide-label'
            ),
            Column(
                Field('period'),
                css_class='small-12 medium-3 hide-label'
            ),
            Column(
                ButtonHolder(
                    Submit(
                        'submit',
                        _('Save'),
                        css_class='tiny expand',
                    ),
                    css_class='text-right',
                ),
                css_class='small-12 medium-2'
            ),
            css_class='collapse',
        ),
    )
    
    return helper

def year_helper(form_tag=True):
    """
    Datebook's year form layout helper
    """
    helper = FormHelper()
    helper.form_action = '.'
    helper.form_class = "hide-label right clearfix"
    helper.attrs = {'data_abide': ''}
    helper.form_tag = form_tag
    
    # Build the full layout
    helper.layout = Layout(
        Field(
            'year',
            wrapper_class='left',
        ),
        Submit(
            'submit',
            _('New year'),
            css_class='tiny',
        ),
    )
    
    return helper

def notes_helper(form_tag=True):
    """
    Datebook's notes form layout helper
    """
    helper = FormHelper()
    helper.form_action = '.'
    helper.attrs = {'data_abide': ''}
    helper.form_tag = form_tag
    
    # Build the full layout
    helper.layout = Layout(
        SimpleRowColumn('notes', wrapper_class='hide-label'),
        SimpleRowColumn(
            ButtonHolderPanel(
                Submit(
                    'submit',
                    _('Submit'),
                ),
                css_class='text-right',
            ),
        )
    )
    
    return helper

def assign_daymodel_helper(form_tag=True, form_action='.', **kwargs):
    """
    AssignDayModelForm form layout helper
    """
    helper = FormHelper()
    helper.form_action = form_action
    #helper.attrs = {'data_abide': ''}
    helper.form_tag = form_tag
    helper.form_id = "daymodel-menu-chooser-form"
    
    # Build the full layout
    helper.layout = Layout(
        Row(
            Column(
                Field(
                    'days',
                    wrapper_class='hide',
                ),
                Field(
                    'daymodel',
                    wrapper_class='hide-label',
                ),
                css_class='medium-9'
            ),
            Column(
                Submit(
                    'submit',
                    _('Ok'),
                    css_class='postfix',
                ),
                css_class='medium-3'
            ),
            css_class='collapse postfix-round',
        ),
        SimpleRowColumn('with_content', wrapper_class='text-center'),
    )
    
    return helper



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Datebook'
        db.create_table(u'datebook_datebook', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('author', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('created', self.gf('django.db.models.fields.DateTimeField')(blank=True)),
            ('modified', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('period', self.gf('django.db.models.fields.DateField')()),
        ))
        db.send_create_signal(u'datebook', ['Datebook'])

        # Adding unique constraint on 'Datebook', fields ['author', 'period']
        db.create_unique(u'datebook_datebook', ['author_id', 'period'])

        # Adding model 'DayEntry'
        db.create_table(u'datebook_dayentry', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('datebook', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['datebook.Datebook'])),
            ('activity_date', self.gf('django.db.models.fields.DateField')()),
            ('vacation', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('content', self.gf('django.db.models.fields.TextField')(max_length=500, blank=True)),
            ('start', self.gf('django.db.models.fields.DateTimeField')()),
            ('stop', self.gf('django.db.models.fields.DateTimeField')()),
            ('pause', self.gf('django.db.models.fields.TimeField')(default=datetime.time(0, 0))),
        ))
        db.send_create_signal(u'datebook', ['DayEntry'])

        # Adding unique constraint on 'DayEntry', fields ['datebook', 'activity_date']
        db.create_unique(u'datebook_dayentry', ['datebook_id', 'activity_date'])


    def backwards(self, orm):
        # Removing unique constraint on 'DayEntry', fields ['datebook', 'activity_date']
        db.delete_unique(u'datebook_dayentry', ['datebook_id', 'activity_date'])

        # Removing unique constraint on 'Datebook', fields ['author', 'period']
        db.delete_unique(u'datebook_datebook', ['author_id', 'period'])

        # Deleting model 'Datebook'
        db.delete_table(u'datebook_datebook')

        # Deleting model 'DayEntry'
        db.delete_table(u'datebook_dayentry')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'datebook.datebook': {
            'Meta': {'unique_together': "(('author', 'period'),)", 'object_name': 'Datebook'},
            'author': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"}),
            'created': ('django.db.models.fields.DateTimeField', [], {'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'modified': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'period': ('django.db.models.fields.DateField', [], {})
        },
        u'datebook.dayentry': {
            'Meta': {'unique_together': "(('datebook', 'activity_date'),)", 'object_name': 'DayEntry'},
            'activity_date': ('django.db.models.fields.DateField', [], {}),
            'content': ('django.db.models.fields.TextField', [], {'max_length': '500', 'blank': 'True'}),
            'datebook': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['datebook.Datebook']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'pause': ('django.db.models.fields.TimeField', [], {'default': 'datetime.time(0, 0)'}),
            'start': ('django.db.models.fields.DateTimeField', [], {}),
            'stop': ('django.db.models.fields.DateTimeField', [], {}),
            'vacation': ('django.db.models.fields.BooleanField', [], {'default': 'False'})
        }
    }

    complete_apps = ['datebook']


# -*- coding: utf-8 -*-
"""
Datebook month views
"""
import datetime

from django.conf import settings
from django import http
from django.views import generic
from django.views.generic.edit import FormMixin
from django.contrib.auth.models import User

from braces.views import LoginRequiredMixin, PermissionRequiredMixin

from datebook.forms.month import DatebookForm, DatebookNotesForm
from datebook.forms.daymodel import AssignDayModelForm
from datebook.models import Datebook
from datebook.mixins import DatebookCalendarMixin, DatebookCalendarAutoCreateMixin, OwnerOrPermissionRequiredMixin
from datebook.utils import format_seconds_to_clock, get_day_weekno

class DatebookMonthFormView(PermissionRequiredMixin, generic.FormView):
    """
    Datebook create form view
    """
    model = Datebook
    form_class = DatebookForm
    template_name = 'datebook/month/form.html'
    permission_required = 'datebook.add_datebook'
    raise_exception = True

    def get_form(self, form_class):
        excluded_users = Datebook.objects.all().values('author_id').distinct()
        self.available_users = User.objects.all().exclude(pk__in=[v['author_id'] for v in excluded_users]).order_by('username')
        
        if self.available_users.count() == 0:
            return None
        return super(DatebookMonthFormView, self).get_form(form_class)
        
    def get_form_kwargs(self, **kwargs):
        kwargs = super(DatebookMonthFormView, self).get_form_kwargs(**kwargs)
        kwargs.update({
            'available_users': self.available_users,
        })
        return kwargs
    
    def form_valid(self, form):
        self.object = form.save()
        return super(DatebookMonthFormView, self).form_valid(form)
    
    def get_success_url(self):
        return self.object.get_absolute_url()


class DatebookMonthGetOrCreateView(DatebookCalendarMixin, OwnerOrPermissionRequiredMixin, generic.View):
    """
    Automatically create the datebook if it does not allready exists for the "author+year+month" 
    kwargs given, then redirect to its page.
    
    If the Datebook allready exists for the given kwargs, raise a "Http404"
    """
    permission_required = 'datebook.add_datebook'
    raise_exception = True
    
    def get(self, request, *args, **kwargs):
        if Datebook.objects.filter(author=self.author, period__year=self.year, period__month=self.month).count()>0:
            raise http.Http404
        
        d = self.author.datebook_set.create(period=datetime.date(year=self.year, month=self.month, day=1))
        
        return http.HttpResponseRedirect(d.get_absolute_url())


class DatebookMonthCurrentView(DatebookCalendarAutoCreateMixin, OwnerOrPermissionRequiredMixin, generic.View):
    """
    Automatically create the datebook if it does not allready exists for the "author" 
    kwarg and the current month, then redirect to its page.
    
    If the Datebook allready exists for the given kwargs, directly redirect to it
    """
    def get(self, request, *args, **kwargs):
        self.get_current_date()
        
        q = Datebook.objects.filter(author=self.author, period__year=self.year, period__month=self.month)
        
        if q.count()>0:
            d = q.order_by('period')[0:1][0]
        else:
            d = self.author.datebook_set.create(period=datetime.date(year=self.year, month=self.month, day=1))
        
        return http.HttpResponseRedirect(d.get_absolute_url())

class DatebookMonthView(LoginRequiredMixin, DatebookCalendarMixin, FormMixin, generic.TemplateView):
    """
    Datebook month details view
    
    Get the Calendar for the given year+month then fill it with day entries
    
    Accept POST request for the AssignDayModelForm form that fill days from a day model.
    """
    template_name = "datebook/month/calendar.html"
    form_class = AssignDayModelForm

    def get_calendar(self, day_filters={}):
        """
        Where we get the Datebook's calendar and crawl it to compute some 
        values about its weeks and days
        """
        current_day = datetime.date.today()
        
        # Init the calendar object
        _cal = super(DatebookMonthView, self).get_calendar()
        
        # Get month weeks structure, removing days that are not month's days (equal to 0)
        week_days = [filter(None, item) for item in _cal.monthdayscalendar(self.object.period.year, self.object.period.month)]
        weeks_totals = [{'current': False, 'active': False, 'elapsed_seconds':0, 'overtime_seconds':0, 'vacations':0} for i in range(0, len(week_days))]
        # Tag the current week if we are on a current month
        if current_day.year == self.object.period.year and current_day.month == self.object.period.month:
            for i, item in enumerate(weeks_totals, start=0):
                if current_day.day in week_days[i]:
                    item['current'] = True
                    break
        
        # Calculate total elapsed time for worked days and total vacations
        day_entries = self.get_dayentry_list(day_filters)
        total_elapsed_seconds = total_overtime_seconds = total_vacation = 0
        
        for item in day_entries:
            # Find the day's week number
            weekno = get_day_weekno(week_days, item.activity_date.day)
            week = weeks_totals[weekno]
            
            # Default value for day's mark used in calendar template for a day equal or after the current day
            item.projected = False
            
            # Do not calculate future days (from current day and further)
            if current_day <= item.activity_date:
                item.projected = True
                continue
            # Mark the day's week as active (the week have days that are not projections)
            else:
                weeks_totals[weekno]['active'] = True
            
            # Do not calculate vacations days
            if item.vacation:
                total_vacation += 1
                weeks_totals[weekno]['vacations'] += 1
                continue
            
            # Compute totals (for months and weeks)
            total_elapsed_seconds += item.get_elapsed_seconds()
            total_overtime_seconds += item.get_overtime_seconds()
            weeks_totals[weekno]['elapsed_seconds'] += item.get_elapsed_seconds()
            weeks_totals[weekno]['overtime_seconds'] += item.get_overtime_seconds()
        
        
        # Post process week totals for some additional values
        for item in weeks_totals:
            item['elapsed_time'] = format_seconds_to_clock(item['elapsed_seconds'])
            item['overtime_time'] = format_seconds_to_clock(item['overtime_seconds'])
            
        
        calendar_datas = {
            "days": [item.day for item in _cal.itermonthdates(self.object.period.year, self.object.period.month) if item.month == self.object.period.month],
            "weekheader": _cal.formatweekheader(),
            "weeks_totals": weeks_totals,
            "month": _cal.formatmonth(self.object.period.year, self.object.period.month, dayentries=day_entries, current_day=current_day),
            "elapsed_seconds": 0,
            "elapsed_time": None,
            "overtime_seconds": 0,
            "overtime_time": None,
            "vacations": 0,
        }
        
        # Crawl all weeks to calculate month totals
        for item in weeks_totals:
            calendar_datas['elapsed_seconds'] += item['elapsed_seconds']
            calendar_datas['overtime_seconds'] += item['overtime_seconds']
            calendar_datas['vacations'] += item['vacations']
            
        calendar_datas['elapsed_time'] = format_seconds_to_clock(calendar_datas['elapsed_seconds'])
        calendar_datas['overtime_time'] = format_seconds_to_clock(calendar_datas['overtime_seconds'])
        
        return calendar_datas
    
    def get_day_models(self):
        """
        Get and return author's day models
        """
        return self.author.daymodel_set.all().order_by('title').values('id', 'title')
        
    def get_context_data(self, **kwargs):
        context = super(DatebookMonthView, self).get_context_data(**kwargs)
        context.update({
            'datebook': self.object,
            'daymodels_form': self.form,
            'datebook_calendar': self.calendar,
            'day_models': self.get_day_models(),
            'DATEBOOK_TEXT_MARKUP_RENDER_TEMPLATE': settings.DATEBOOK_TEXT_MARKUP_RENDER_TEMPLATE,
        })
        return context
        
    def get_form_kwargs(self, **kwargs):
        kwargs = super(DatebookMonthView, self).get_form_kwargs(**kwargs)
        kwargs.update({
            'author': self.author,
            'datebook': self.object,
            'daychoices': self.calendar['days'],
        })
        return kwargs

    def form_valid(self, form):
        """
        If the form is valid, save the associated model.
        """
        form.save()
        return super(DatebookMonthView, self).form_valid(form)
 
    def get_success_url(self):
        return self.object.get_absolute_url()
   
    def get(self, request, *args, **kwargs):
        self.object = self.get_datebook({'period__year': self.year, 'period__month': self.month})
        self.calendar = self.get_calendar()
        
        form_class = self.get_form_class()
        self.form = self.get_form(form_class)
        
        return self.render_to_response(self.get_context_data(**kwargs))
    
    def post(self, request, *args, **kwargs):
        self.object = self.get_datebook({'period__year': self.year, 'period__month': self.month})
        self.calendar = self.get_calendar()
        
        form_class = self.get_form_class()
        self.form = self.get_form(form_class)
        
        if self.form.is_valid():
            return self.form_valid(self.form)
        else:
            return self.form_invalid(self.form)


class DatebookNotesFormView(DatebookCalendarMixin, OwnerOrPermissionRequiredMixin, generic.UpdateView):
    """
    Datebook create form view
    """
    model = Datebook
    form_class = DatebookNotesForm
    template_name = 'datebook/month/notes_form.html'
    permission_required = 'datebook.change_datebook'
    raise_exception = True
    
    def get_object(self):
        self.datebook = self.get_datebook({'period__year': self.year, 'period__month': self.month})
        self.author = self.datebook.author
        return self.datebook
    
    def get_success_url(self):
        return self.object.get_absolute_url()
        
    def get_context_data(self, **kwargs):
        context = super(DatebookNotesFormView, self).get_context_data(**kwargs)
        context.update({
            'DATEBOOK_TEXT_FIELD_JS_TEMPLATE': settings.DATEBOOK_TEXT_FIELD_JS_TEMPLATE,
            'DATEBOOK_TEXT_MARKUP_RENDER_TEMPLATE': settings.DATEBOOK_TEXT_MARKUP_RENDER_TEMPLATE,
        })
        return context



# -*- coding: utf-8 -*-
"""
Root url's map for application
"""
from django.conf.urls import *

from datebook.views import IndexView
from datebook.views.author import DatebookAuthorView
from datebook.views.year import DatebookYearView
from datebook.views.month import DatebookMonthView, DatebookMonthGetOrCreateView, DatebookMonthCurrentView, DatebookMonthFormView, DatebookNotesFormView
from datebook.views.day import DayEntryFormCreateView, DayEntryDetailView, DayEntryFormEditView, DayEntryCurrentView, DayEntryDeleteFormView
from datebook.views.daymodel import DayModelListView, DayEntryToDayModelFormView, DayModelFormEditView

urlpatterns = patterns('',
    url(r'^$', IndexView.as_view(), name='index'),
    
    url(r'^create/$', DatebookMonthFormView.as_view(), name='create'),
    
    url(r'^(?P<author>\w+)/$', DatebookAuthorView.as_view(), name='author-detail'),
    
    url(r'^(?P<author>\w+)/day-models/$', DayModelListView.as_view(), name='day-models'),
    url(r'^(?P<author>\w+)/day-models/(?P<pk>\d+)/$', DayModelFormEditView.as_view(), name='day-model-edit'),
    
    url(r'^(?P<author>\w+)/current-day/$', DayEntryCurrentView.as_view(), name='current-day'),
    url(r'^(?P<author>\w+)/current-month/$', DatebookMonthCurrentView.as_view(), name='current-month'),
    
    url(r'^(?P<author>\w+)/(?P<year>\d{4})/$', DatebookYearView.as_view(), name='year-detail'),
    url(r'^(?P<author>\w+)/(?P<year>\d{4})/add/(?P<month>\d{1,2})/$', DatebookMonthGetOrCreateView.as_view(), name='month-add'),
    
    url(r'^(?P<author>\w+)/(?P<year>\d{4})/(?P<month>\d{1,2})/$', DatebookMonthView.as_view(), name='month-detail'),
    url(r'^(?P<author>\w+)/(?P<year>\d{4})/(?P<month>\d{1,2})/notes/$', DatebookNotesFormView.as_view(), name='month-notes'),
    
    url(r'^(?P<author>\w+)/(?P<year>\d{4})/(?P<month>\d{1,2})/add/(?P<day>\d{1,2})/$', DayEntryFormCreateView.as_view(), name='day-add'),
    
    url(r'^(?P<author>\w+)/(?P<year>\d{4})/(?P<month>\d{1,2})/(?P<day>\d{1,2})/$', DayEntryDetailView.as_view(), name='day-detail'),
    url(r'^(?P<author>\w+)/(?P<year>\d{4})/(?P<month>\d{1,2})/(?P<day>\d{1,2})/edit/$', DayEntryFormEditView.as_view(), name='day-edit'),
    url(r'^(?P<author>\w+)/(?P<year>\d{4})/(?P<month>\d{1,2})/(?P<day>\d{1,2})/remove/$', DayEntryDeleteFormView.as_view(), name='day-remove'),
    url(r'^(?P<author>\w+)/(?P<year>\d{4})/(?P<month>\d{1,2})/(?P<day>\d{1,2})/to-daymodel/$', DayEntryToDayModelFormView.as_view(), name='dayentry-to-daymodel'),
)



# -*- coding: utf-8 -*-
"""
Model admin
"""
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from models import *

class DatebookAdmin(admin.ModelAdmin):
    ordering = ('-modified',)
    list_filter = ('created', 'modified', 'author')
    list_display = ('period_title', 'author', 'days_count', 'created', 'modified')
    raw_id_fields = ("author",)

    def period_title(self, datebook):
        return datebook
    period_title.short_description = _("month of activity")
    period_title.admin_order_field = 'period'

    def days_count(self, datebook):
        return datebook.dayentry_set.all().count()
    days_count.short_description = _('Days')

class DayBaseAdmin(admin.ModelAdmin):
    def start_time(self, day):
        return day.start.time()
    start_time.short_description = _("start")
    start_time.admin_order_field = 'start'

    def stop_time(self, day):
        return day.stop.time()
    stop_time.short_description = _("stop")
    stop_time.admin_order_field = 'stop'


class DayEntryAdmin(DayBaseAdmin):
    ordering = ('-activity_date',)
    list_filter = ('vacation', 'datebook__author', 'activity_date')
    list_display = ('activity_date', 'datebook_title', 'start_time', 'stop_time', 'pause', 'overtime', 'vacation')
    list_display_links = ('activity_date',)
    raw_id_fields = ("datebook",)
    fieldsets = (
        (_('Date'), {
            'fields': ('datebook', 'activity_date')
        }),
        (_('Time'), {
            'fields': ('vacation', 'start', 'stop', 'pause', 'overtime')
        }),
        (_('Content'), {
            'fields': ('content',),
        }),
    )

    def datebook_title(self, day):
        return "{author} - {period}".format(author=day.datebook.author, period=day.datebook)
    datebook_title.short_description = _("Datebook")
    datebook_title.admin_order_field = 'datebook__author__username'


class DayModelAdmin(DayBaseAdmin):
    ordering = ('author__username', 'title',)
    list_filter = ('author',)
    list_display = ('title', 'author', 'start_time', 'stop_time', 'pause', 'overtime')
    raw_id_fields = ("author",)
    fieldsets = (
        (_('Parameters'), {
            'fields': ('author', 'title')
        }),
        (_('Time'), {
            'fields': ('start', 'stop', 'pause', 'overtime')
        }),
        (_('Content'), {
            'fields': ('content',),
        }),
    )


admin.site.register(Datebook, DatebookAdmin)
admin.site.register(DayEntry, DayEntryAdmin)
admin.site.register(DayModel, DayModelAdmin)



"""
Some markup utilities for RST and DjangoCodeMirror usage
"""
from django.forms import ValidationError

from rstview.parser import SourceReporter, map_parsing_errors

from djangocodemirror.fields import DjangoCodeMirrorField

def get_text_field(form_instance, **kwargs):
    """
    Return a DjangoCodeMirrorField field
    """
    kwargs.update({
        'config_name': 'datebook'
    })
    return DjangoCodeMirrorField(**kwargs)


def clean_restructuredtext(form_instance, content):
    """
    RST syntax validation
    """
    if content:
        errors = SourceReporter(content)
        if errors:
            raise ValidationError(map(map_parsing_errors, errors))
    return content



"""
Default Datebook settings to import/define in your project settings
"""

# Add new specific "rstview" parser settings for Forum app, if you have other apps 
# that define parser settings this can lead to overwrite problems. In this 
# case, just define all parser setting in 'RSTVIEW_PARSER_FILTER_SETTINGS' in 
# the same settings file.
RSTVIEW_PARSER_FILTER_SETTINGS = {
    'datebook':{
        'initial_header_level': 5,
        'file_insertion_enabled': False,
        'raw_enabled': False,
        'footnote_references': 'superscript',
        'doctitle_xform': False,
    },
}

#
# Optionnal text markup settings
#

# Field helper for texts in forms
DATEBOOK_TEXT_FIELD_HELPER_PATH = None # Default, just a CharField
#DATEBOOK_TEXT_FIELD_HELPER_PATH = "datebook.markup.get_text_field" # Use DjangoCodeMirror

# Validator helper for texts in forms
DATEBOOK_TEXT_VALIDATOR_HELPER_PATH = None # Default, no markup validation
#DATEBOOK_TEXT_VALIDATOR_HELPER_PATH = "datebook.markup.clean_restructuredtext" # Validation for RST syntax (with Rstview)

# Text markup renderer
DATEBOOK_TEXT_MARKUP_RENDER_TEMPLATE = None # Default, just a CharField
#DATEBOOK_TEXT_MARKUP_RENDER_TEMPLATE = "datebook/markup/_text_markup_render.html" # Use Rstview renderer

# Template to init some Javascript for texts in forms
DATEBOOK_TEXT_FIELD_JS_TEMPLATE = None # Default, no JS template
#DATEBOOK_TEXT_FIELD_JS_TEMPLATE = "datebook/markup/_text_field_djangocodemirror_js.html" # Use DjangoCodeMirror



# -*- coding: utf-8 -*-
"""
Forms for day forms
"""
import datetime

from django import forms
from django.utils.translation import ugettext as _
from django.utils.timezone import now as tz_now, make_aware, utc

from datebook.models import DayEntry, DayModel
from datebook.forms.day import DATETIME_FORMATS, DayBaseFormMixin
from datebook.forms import CrispyFormMixin

class DayToDayModelForm(DayBaseFormMixin, CrispyFormMixin, forms.ModelForm):
    """
    DayModel form
    """
    crispy_form_helper_kwargs = {'day_to_model_mode': True}
    #crispy_form_helper_path = 'datebook.forms.crispies.day_helper'

    def __init__(self, author, *args, **kwargs):
        self.author = author
        self.dayentry = kwargs.pop('dayentry', None)
        self.daydate = kwargs.pop('day_date', None)
        
        # Args to give to the form layout method
        self.crispy_form_helper_kwargs.update({
            'form_action': kwargs.pop('form_action'),
        })
        
        # Fill initial datas
        kwargs = self.fill_initial_data(*args, **kwargs)
        
        super(DayToDayModelForm, self).__init__(*args, **kwargs)
        super(forms.ModelForm, self).__init__(*args, **kwargs)
        
        # Init some special fields
        kwargs = self.init_fields(*args, **kwargs)
    
    def save(self, *args, **kwargs):
        instance = super(DayToDayModelForm, self).save(commit=False, *args, **kwargs)
        instance.start = self.cleaned_data['start_datetime']
        instance.stop = self.cleaned_data['stop_datetime']
        instance.author = self.author
        instance.save()
        
        return instance
    
    class Meta:
        model = DayModel
        exclude = ('author', 'start', 'stop')
        widgets = {
            'pause': forms.TimeInput(format=DATETIME_FORMATS['input_time_formats'][0]),
            'overtime': forms.TimeInput(format=DATETIME_FORMATS['input_time_formats'][0]),
        }


class AssignDayModelForm(CrispyFormMixin, forms.Form):
    """
    Form to fill selected DayEntry with contents from a DayModel
    """
    crispy_form_helper_path = 'datebook.forms.crispies.assign_daymodel_helper'
    
    def __init__(self, *args, **kwargs):
        self.author = kwargs.pop('author')
        self.datebook = kwargs.pop('datebook')
        self.daydate = self.datebook.period
        self.daychoices = [(item, str(item)) for item in kwargs.pop('daychoices')]
        self.daymodels_queryset = self.author.daymodel_set.all().order_by('title')
        
        super(AssignDayModelForm, self).__init__(*args, **kwargs)
        super(forms.Form, self).__init__(*args, **kwargs)
        
        # Init fields
        self.fields['days'] = forms.MultipleChoiceField(choices=self.daychoices, required=True)
        self.fields['daymodel'] = forms.ModelChoiceField(queryset=self.daymodels_queryset, required=True)
        self.fields['with_content'] = forms.BooleanField(label=_('Use the model\'s content text'), required=False)

    def combine_day_and_daymodel_time(self, day_start, daymodel_start, daymodel_delta):
        """
        Combine the day date with the daymodel start time, then calcul the stop 
        from the start date and the given delta time (between daymodel start and 
        stop time)
        
        This way we have clean datetimes, correctly calculated with the right 
        day date and daymodel time.
        
        Return the calculated start and stop datetimes
        """
        if not isinstance(day_start, datetime.date):
            day_start = day_start.date()
            
        start = make_aware(datetime.datetime.combine(day_start, daymodel_start.time()), utc)
        return (
            start,
            start+daymodel_delta,
        )

    def save(self, *args, **kwargs):
        daymodel = self.cleaned_data['daymodel']
        with_content = self.cleaned_data['with_content']
        daymodel_delta = daymodel.stop - daymodel.start
        # Bind datetime for each days using datebook period as the base date
        daydates = [self.daydate.replace(day=int(item)) for item in self.cleaned_data['days']]
        
        # Fill existing entries
        for entry in self.datebook.dayentry_set.filter(activity_date__in=daydates).order_by('activity_date'):
            # Get the start/stop datetimes
            goto_start, goto_stop = self.combine_day_and_daymodel_time(entry.start, daymodel.start, daymodel_delta)
            # Fill object attribute using the daymodel
            entry.start = goto_start
            entry.stop = goto_stop
            entry.pause = daymodel.pause
            entry.overtime = daymodel.overtime
            if with_content:
                entry.content = daymodel.content
            entry.vacation = False # Allways remove the vacation
            entry.save()
            # Remove the day number from remaining selected days
            i = self.cleaned_data['days'].index(str(entry.activity_date.day))
            self.cleaned_data['days'].pop(i)
        
        # Create remaining selected days
        new_days = []
        for day_no in self.cleaned_data['days']:
            activity_date = self.datebook.period.replace(day=int(day_no))

            goto_start, goto_stop = self.combine_day_and_daymodel_time(activity_date, daymodel.start, daymodel_delta)
            
            content = ""
            if with_content:
                content = daymodel.content
            
            new_days.append(DayEntry(
                datebook=self.datebook,
                activity_date=activity_date,
                start=goto_start,
                stop=goto_stop,
                pause=daymodel.pause,
                overtime=daymodel.overtime,
                content=content,
                vacation=False,
            ))
        # Bulk create all new days
        if new_days:
            DayEntry.objects.bulk_create(new_days)
        # Update the datebook because model save method is not triggered with bulk creating
        self.datebook.modified = tz_now()
        self.datebook.save()
        
        return None



# -*- coding: utf-8 -*-
"""
Forms for month forms
"""
from django.conf import settings
from django import forms
from django.utils.translation import ugettext as _

from crispy_forms.helper import FormHelper

from datebook.models import Datebook
from datebook.forms import CrispyFormMixin
from datebook.utils.imports import safe_import_module

class DatebookForm(CrispyFormMixin, forms.Form):
    """
    Datebook form
    """
    crispy_form_helper_path = 'datebook.forms.crispies.month_helper'
    
    def __init__(self, *args, **kwargs):
        self.available_users = kwargs.pop('available_users', [])
        super(DatebookForm, self).__init__(*args, **kwargs)
        super(forms.Form, self).__init__(*args, **kwargs)
        
        self.fields['owner'] = forms.ModelChoiceField(label=_('owner'), queryset=self.available_users, empty_label=None)
        self.fields['period'] = forms.DateField(label=_('period'))
    
    def clean_period(self):
        period = self.cleaned_data['period']
        period = period.replace(day=1)
        
        return period
    
    def save(self, *args, **kwargs):
        instance = Datebook(
            author=self.cleaned_data['owner'],
            period=self.cleaned_data['period']
        )
        instance.save()
        
        return instance


class DatebookNotesForm(CrispyFormMixin, forms.ModelForm):
    """
    Datebook form
    """
    crispy_form_helper_path = 'datebook.forms.crispies.notes_helper'
    
    def __init__(self, *args, **kwargs):
        super(DatebookNotesForm, self).__init__(*args, **kwargs)
        super(forms.ModelForm, self).__init__(*args, **kwargs)
        
        # Set the form field for Datebook.notes
        field_helper = safe_import_module(settings.DATEBOOK_TEXT_FIELD_HELPER_PATH)
        if field_helper is not None:
            self.fields['notes'] = field_helper(self, **{'label':_('notes'), 'required':True})
    
    def clean_notes(self):
        """
        Text content validation
        """
        notes = self.cleaned_data.get("notes")
        validation_helper = safe_import_module(settings.DATEBOOK_TEXT_VALIDATOR_HELPER_PATH)
        if validation_helper is not None:
            return validation_helper(self, notes)
        else:
            return notes
    
    class Meta:
        model = Datebook
        fields = ['notes']



# -*- coding: utf-8 -*-
"""
Forms for day forms
"""
from django.conf import settings
from django import forms
from django.utils.translation import ugettext as _

from arrow import Arrow

from datebook.models import DayEntry
from datebook.forms import CrispyFormMixin
from datebook.utils.imports import safe_import_module

DATETIME_FORMATS = {
    'input_date_formats': ['%d/%m/%Y'],
    'input_time_formats': ['%H:%M'], 
    'widget': forms.SplitDateTimeWidget(date_format='%d/%m/%Y', time_format='%H:%M'),
}

class DayBaseFormMixin(object):
    """
    DayBase form mixin
    """
    crispy_form_helper_path = 'datebook.forms.crispies.day_helper'
    crispy_form_helper_kwargs = {}
    
    def fill_initial_data(self, *args, **kwargs):
        # Pass initial data for start and stop to their SplitDateTimeField clones
        if 'start' in kwargs['initial']:
            kwargs['initial']['start_datetime'] = kwargs['initial']['start']
        if 'stop' in kwargs['initial']:
            kwargs['initial']['stop_datetime'] = kwargs['initial']['stop']
            
        # For existing instance (in edit mode) pass the start and stop values to their 
        # clone with SplitDateTimeField via initial datas
        if kwargs.get('instance'):
            kwargs['initial']['start_datetime'] = kwargs['instance'].start
            kwargs['initial']['stop_datetime'] = kwargs['instance'].stop
        
        return kwargs
    
    def init_fields(self, *args, **kwargs):
        self.fields['start_datetime'] = forms.SplitDateTimeField(label=_('start'), **DATETIME_FORMATS)
        self.fields['stop_datetime'] = forms.SplitDateTimeField(label=_('stop'), **DATETIME_FORMATS)
        
        # Set the form field for DayEntry.content
        field_helper = safe_import_module(settings.DATEBOOK_TEXT_FIELD_HELPER_PATH)
        if field_helper is not None:
            self.fields['content'] = field_helper(self, **{'label':_('content'), 'required': False})
    
    def clean_content(self):
        """
        Text content validation
        """
        content = self.cleaned_data.get("content")
        validation_helper = safe_import_module(settings.DATEBOOK_TEXT_VALIDATOR_HELPER_PATH)
        if validation_helper is not None:
            return validation_helper(self, content)
        else:
            return content
    
    def clean_start_datetime(self):
        start = self.cleaned_data['start_datetime']
        # Day entry can't start before the targeted day date
        if start and start.date() < self.daydate:
            raise forms.ValidationError(_("You can't start a day before itself"))
        # Day entry can't start after the targeted day date
        if start and start.date() > self.daydate:
            raise forms.ValidationError(_("You can't start a day after itself"))
        
        return start
    
    def clean_stop_datetime(self):
        start = self.cleaned_data.get('start_datetime')
        stop = self.cleaned_data['stop_datetime']
        # Day entry can't stop before the start
        if start and stop and stop <= start:
            raise forms.ValidationError(_("Stop time can't be less or equal to start time"))
        # Day entry can't stop in more than one futur day from the targeted day date
        if stop and stop.date() > Arrow.fromdate(self.daydate).replace(days=1).date():
            raise forms.ValidationError(_("Stop time can't be more than the next day"))
        
        return stop
    
    # TODO: overtime must not be more than effective worked time
    #def clean_overtime(self):
        #overtime = self.cleaned_data.get('overtime')
        #return overtime
    
    # TODO
    #def clean_pause(self):
        #start = self.cleaned_data.get('start_datetime')
        #stop = self.cleaned_data.get('stop_datetime')
        #pause = self.cleaned_data['pause']
        ## Pause time can't be more than elapsed time between start and stop
        #if start and stop and pause and False:
            #raise forms.ValidationError("Pause time is more than the elapsed time")
        
        #return pause
    

class DayEntryForm(DayBaseFormMixin, CrispyFormMixin, forms.ModelForm):
    """
    DayEntry form
    """
    def __init__(self, datebook, day, *args, **kwargs):
        self.datebook = datebook
        self.daydate = datebook.period.replace(day=day)
        
        # Args to give to the form layout method
        self.crispy_form_helper_kwargs.update({
            'next_day': kwargs.pop('next_day', None),
            'day_to_model_url': kwargs.pop('day_to_model_url', None),
            'form_action': kwargs.pop('form_action'),
            'remove_url': kwargs.pop('remove_url', None),
        })
        
        # Fill initial datas
        kwargs = self.fill_initial_data(*args, **kwargs)
        
        super(DayEntryForm, self).__init__(*args, **kwargs)
        super(forms.ModelForm, self).__init__(*args, **kwargs)
        
        # Init some special fields
        kwargs = self.init_fields(*args, **kwargs)
    
    def clean(self):
        cleaned_data = super(DayBaseFormMixin, self).clean()
        content = cleaned_data.get("content")
        vacation = cleaned_data.get("vacation")
        # Content text is only required when vacation is not checked
        if not vacation and not content:
            raise forms.ValidationError(_("Worked days require a content text"))
        
        return cleaned_data
    
    def save(self, *args, **kwargs):
        instance = super(DayEntryForm, self).save(commit=False, *args, **kwargs)
        instance.start = self.cleaned_data['start_datetime']
        instance.stop = self.cleaned_data['stop_datetime']
        instance.datebook = self.datebook
        instance.activity_date = self.daydate
        instance.save()
        
        return instance
    
    class Meta:
        model = DayEntry
        exclude = ('datebook', 'activity_date', 'start', 'stop')
        widgets = {
            'pause': forms.TimeInput(format=DATETIME_FORMATS['input_time_formats'][0]),
            'overtime': forms.TimeInput(format=DATETIME_FORMATS['input_time_formats'][0]),
        }


class DayEntryCreateForm(DayEntryForm):
    def clean(self):
        cleaned_data = super(DayEntryCreateForm, self).clean()
        
        # Validate that there is not allready a day entry for the same day
        try:
            obj = DayEntry.objects.get(datebook=self.datebook, activity_date=self.daydate)
        except DayEntry.DoesNotExist:
            pass
        else:
            raise forms.ValidationError(_("This day entry has allready been created"))
            
        return cleaned_data



# -*- coding: utf-8 -*-
"""
Modèles de données
"""
import datetime

from django.db import models
from django.contrib.auth.models import User
from django.utils.translation import ugettext_lazy as _
from django.utils.timezone import now as tz_now, localtime

from datebook import utils

class Datebook(models.Model):
    """
    Datebook is only for one user (its author) and it is not shared.
    
    A Datebook is for a specific month of a year.
    
    A Datebook is alike "lazy" in the way that its 'DayEntry' are only created 
    when the user fill them and so Datebook are not initialized with all its 
    month's days when created.
    """
    author = models.ForeignKey(User, verbose_name=_('author'))
    created = models.DateTimeField(_('created'), blank=True, editable=False)
    modified = models.DateTimeField(_('last edit'), auto_now=True)
    period = models.DateField(_("month of activity"), blank=False)
    notes = models.TextField(_("content"), max_length=500, blank=True)

    def __unicode__(self):
        return self.period.strftime("%B %Y")

    @models.permalink
    def get_absolute_url(self):
        return ('datebook:month-detail', (), {
            'author': self.author.username,
            'year': self.period.year,
            'month': self.period.strftime('%m'),
        })

    def clean(self):
        """Allways forcing to the first day of the month"""
        self.period = self.period.replace(day=1) 

    def save(self, *args, **kwargs):
        # First create
        if not self.created:
            self.created = tz_now()
        
        super(Datebook, self).save(*args, **kwargs)

    class Meta:
        unique_together = ("author", "period")
        verbose_name = _("Datebook")
        verbose_name_plural = _("Datebooks")


class DayBase(models.Model):
    """
    Base day model
    """
    content = models.TextField(_("content"), max_length=500, blank=True)

    start = models.DateTimeField(_('start'), blank=False) # should default to something like 9h with the dayentry date
    stop = models.DateTimeField(_('stop'), blank=False) # should default to something like 19h with the dayentry date
    pause = models.TimeField(_('pause'), blank=False, default=datetime.time(0, 0))
    overtime = models.TimeField(_('overtime'), blank=False, default=datetime.time(0, 0))

    def get_display_hour(self, timeobj):
        """Formating time "XXhYY" display, minutes are not displayed if they are not > 0"""
        _m = ""
        timeobj = localtime(timeobj)
        if timeobj.minute > 0:
            _m = str(timeobj.minute).rjust(2, "0")
        return _('%(hour)sh%(minute)s') % {'hour': str(timeobj.hour).rjust(2, "0"), 'minute': _m}

    def get_working_hours(self):
        """Display working hours like: '08h to 18h59'"""
        return _('%(start)s to %(stop)s') % {'start': self.get_display_hour(self.start), 'stop': self.get_display_hour(self.stop)}

    def get_elapsed_seconds(self):
        """Return elapsed seconds between start and stop, where pause time has been substracted from"""
        return utils.timedelta_to_seconds((self.stop-self.start))-utils.time_to_seconds(self.pause)

    def get_elapsed_time(self):
        """Return formatted clock for elapsed seconds from 'get_elapsed_seconds'"""
        return utils.format_seconds_to_clock(self.get_elapsed_seconds())

    def get_overtime_seconds(self):
        """Return overtime seconds"""
        return utils.time_to_seconds(self.overtime)

    def get_overtime_time(self):
        """Return formatted clock for overtime seconds from 'get_overtime_seconds'"""
        return utils.format_seconds_to_clock(self.get_overtime_seconds())

    #def clean(self):
        ##DEPRECATED: Stuff below are for the form controller
        #from django.core.exceptions import ValidationError
        ## Stop can't be less than start
        #if self.stop < self.start:
            #raise ValidationError(_("Hour stop can't be less than hour start."))
        ## TODO: Pause time can't be superior to the time of (stop-start)
    
    class Meta:
        abstract = True    


class DayEntry(DayBase):
    """
    Activity day in a Datebook
    """
    datebook = models.ForeignKey(Datebook, verbose_name=_('datebook'))
    activity_date = models.DateField(_('activity day date'), blank=False) # inherit month and year from its Datebook
    vacation = models.BooleanField(_('vacation'), default=False, blank=True, null=False)

    def __unicode__(self):
        if not self.activity_date:
            return _("Empty")
        return self.activity_date.strftime("%d/%m/%Y")

    def clean(self):
        # Inherit the year and month from its datebook
        if hasattr(self, 'datebook'):
            self.activity_date = self.activity_date.replace(month=self.datebook.period.month, year=self.datebook.period.year)

    def save(self, *args, **kwargs):
        # Allways update the datebook
        self.datebook.modified = tz_now()
        self.datebook.save()
        
        super(DayEntry, self).save(*args, **kwargs)

    class Meta:
        unique_together = ("datebook", "activity_date")
        verbose_name = _("day entry")
        verbose_name_plural = _("day entries")


class DayModel(DayBase):
    """
    Day model from an user
    """
    author = models.ForeignKey(User, verbose_name=_('author'))
    title = models.CharField(_("title"), blank=False, max_length=255)

    def __unicode__(self):
        return self.title

    class Meta:
        unique_together = ("author", "title")
        verbose_name = _("day model")
        verbose_name_plural = _("day models")



# -*- coding: utf-8 -*-
"""
Day entry views
"""
import datetime
from calendar import TextCalendar

from django.conf import settings
from django.views import generic
from django.core.urlresolvers import reverse
from django.contrib.auth.models import User
from django.shortcuts import get_object_or_404
from django.views.generic.edit import DeleteView
from braces.views import LoginRequiredMixin

from datebook.models import Datebook, DayEntry
from datebook.mixins import AuthorKwargsMixin, DatebookCalendarMixin, OwnerOrPermissionRequiredMixin
from datebook.forms.day import DayEntryForm, DayEntryCreateForm
from datebook.utils import week_from_date

class DayEntryBaseFormView(DatebookCalendarMixin, OwnerOrPermissionRequiredMixin):
    """
    DayEntry base form view
    """
    model = DayEntry
    context_object_name = "dayentry"
    template_name = "datebook/day/form.html"
    ajax_template_name = "datebook/day/form_fragment.html"
    form_class = DayEntryForm
    permission_required = 'datebook.add_dayentry'
    raise_exception = True
    
    def switch_template(self, request):
        """
        Switch to the specific Ajax template if request is detected as an Ajax request
        """
        if request.is_ajax():
            return self.ajax_template_name
        return self.template_name
    
    def get_next_day(self):
        """
        Find if there is possible next day
        
        If the possible next day is not out of range of the month's days, returns its 
        day entry if exists, else will returns a simple 'datetime.date object'. The 
        return will allways be a tuple containing the object and the resolved form's 
        url.
        
        Finally if there is no possible next day, returns None.
        """
        calendar = TextCalendar()
        # Get available days from reduced calendar's datas
        month_days = [item for sublist in calendar.monthdayscalendar(self.year, self.month) for item in sublist if item]
        month_range = (month_days[0], month_days[-1])
        
        # Next day is out of range for current month
        if self.day+1 > month_days[-1]:
            return None
        
        # Next day is in range, try to find if its day entry allready exists
        next_day = datetime.date(self.year, self.month, self.day+1)
        try:
            obj = self.datebook.dayentry_set.get(activity_date=next_day)
        except DayEntry.DoesNotExist:
            return next_day, reverse('datebook:day-add', kwargs={
                'author': self.author,
                'year': next_day.year,
                'month': next_day.month,
                'day': next_day.day,
            })
        else:
            return obj, reverse('datebook:day-edit', kwargs={
                'author': self.author,
                'year': next_day.year,
                'month': next_day.month,
                'day': next_day.day,
            })
        
        return None
    
    def get_context_data(self, **kwargs):
        context = super(DayEntryBaseFormView, self).get_context_data(**kwargs)
        context.update({
            'next_day': self.next_day,
            'DATEBOOK_TEXT_FIELD_JS_TEMPLATE': settings.DATEBOOK_TEXT_FIELD_JS_TEMPLATE,
            'DATEBOOK_TEXT_MARKUP_RENDER_TEMPLATE': settings.DATEBOOK_TEXT_MARKUP_RENDER_TEMPLATE,
        })
        return context

    def get_form(self, form_class):
        """
        Add required args to form instance
        """
        return form_class(self.datebook, self.day, **self.get_form_kwargs())
    
    def get_form_kwargs(self):
        """
        Add post form url
        """
        kwargs = super(DayEntryBaseFormView, self).get_form_kwargs()
        kwargs.update({
            'form_action': '.',
            'next_day': self.next_day,
        })
        return kwargs

    def get_success_url(self):
        """
        If 'submit_and_next' button has been used, redirect to the next day form, 
        else redirect to the month view
        """
        if 'submit_and_next' in self.request.POST:
            if self.next_day:
                return self.next_day[1]
        
        return reverse('datebook:month-detail', kwargs={
            'author': self.author,
            'year': self.object.activity_date.year,
            'month': self.object.activity_date.month,
        })
        
    def get(self, request, *args, **kwargs):
        self.template_name = self.switch_template(request)
        self.datebook = self.get_datebook({'period__year': self.year, 'period__month': self.month})
        self.next_day = self.get_next_day()
        return super(DayEntryBaseFormView, self).get(request, *args, **kwargs)
    
    def post(self, request, *args, **kwargs):
        self.template_name = self.switch_template(request)
        self.datebook = self.get_datebook({'period__year': self.year, 'period__month': self.month})
        self.next_day = self.get_next_day()
        return super(DayEntryBaseFormView, self).post(request, *args, **kwargs)


class DayEntryFormCreateView(DayEntryBaseFormView, generic.CreateView):
    """
    DayEntry form create view
    """
    form_class = DayEntryCreateForm
    
    def get_form_kwargs(self):
        """
        Put the initial data for the datetime start and stop
        
        NOTE: Time start and stop should comes from the User profile settings (date 
              allways come from the datebook period and the required day).
        """
        kwargs = super(DayEntryFormCreateView, self).get_form_kwargs()
        day_date = self.datebook.period.replace(day=self.day)
        time_start = datetime.time(9, 0)
        time_stop = datetime.time(18, 0)
        kwargs.update({
            'initial': {
                'start' : datetime.datetime.combine(day_date, time_start),
                'stop' : datetime.datetime.combine(day_date, time_stop),
                'pause' : datetime.time(1, 0),
            },
            'form_action': reverse('datebook:day-add', kwargs={
                'author': self.author,
                'year': self.year,
                'month': self.month,
                'day': self.day,
            }),
        })
        return kwargs


class DayEntryFormEditView(DayEntryBaseFormView, generic.UpdateView):
    """
    DayEntry form edit view
    """
    permission_required = 'datebook.change_dayentry'
    
    def get_object(self):
        return self.datebook.dayentry_set.get(activity_date__day=self.day)
    
    def get_form_kwargs(self):
        """
        Add post form url
        """
        kwargs = super(DayEntryBaseFormView, self).get_form_kwargs()
        kwargs.update({
            'form_action': reverse('datebook:day-edit', kwargs={
                'author': self.author,
                'year': self.year,
                'month': self.month,
                'day': self.day,
            }),
            'next_day': self.next_day,
            'day_to_model_url': reverse('datebook:dayentry-to-daymodel', kwargs={
                'author': self.author,
                'year': self.year,
                'month': self.month,
                'day': self.day,
            }),
            'remove_url': reverse('datebook:day-remove', kwargs={
                'author': self.author,
                'year': self.year,
                'month': self.month,
                'day': self.day,
            }),
        })
        return kwargs


class DayEntryCurrentView(AuthorKwargsMixin, OwnerOrPermissionRequiredMixin, generic.RedirectView):
    """
    Redirect to the form for the current day. If entry allready exists this will 
    redirect to edit form, else to add form.
    
    If the month datebook does not exist for the current day, this will create it 
    before redirect.
    """
    permission_required = 'datebook.add_dayentry'
    raise_exception = True
    permanent = False
    
    def get_current_date(self):
        """
        Get the current date
        """
        today = datetime.date.today()
        self.year = self.kwargs['year'] = today.year
        self.month = self.kwargs['month'] = today.month
        self.day = self.kwargs['day'] = today.day
        return today
    
    def get_datebook(self):
        """
        Get or Create datebook
        """
        try:
            obj = Datebook.objects.get(author__username=self.kwargs['author'], period__year=self.year, period__month=self.month)
        except Datebook.DoesNotExist:
            self.author = get_object_or_404(User, username=self.kwargs['author'])
            obj = Datebook(author=self.author, period=datetime.date(self.year, self.month, 1))
            obj.save()
        else:
            self.author = obj.author
        return obj
    
    def get_redirect_url(self, author):
        """
        Get Create or Edit dayentry url
        """
        today = self.get_current_date()
        datebook = self.get_datebook()
        
        try:
            obj = DayEntry.objects.get(datebook=datebook, activity_date=datetime.date(self.year, self.month, self.day))
        except DayEntry.DoesNotExist:
            return reverse('datebook:day-add', kwargs={
                'author': self.author,
                'year': self.year,
                'month': self.month,
                'day': self.day,
            })
        else:
            return reverse('datebook:day-edit', kwargs={
                'author': self.author,
                'year': self.year,
                'month': self.month,
                'day': self.day,
            })


class DayEntryDetailView(LoginRequiredMixin, DatebookCalendarMixin, generic.TemplateView):
    """
    DayEntry detail view
    """
    model = DayEntry
    template_name = "datebook/day/detail_fragment.html"
        
    def get_previous_day(self):
        try:
            obj = self.object.get_previous_by_activity_date(**{
                'datebook__author': self.author,
                'activity_date__month': self.object.activity_date.month,
                'activity_date__year': self.object.activity_date.year,
            })
        except DayEntry.DoesNotExist:
            pass
        else:
            return obj
        return None
        
    def get_next_day(self):
        try:
            obj = self.object.get_next_by_activity_date(**{
                'datebook__author': self.author,
                'activity_date__month': self.object.activity_date.month,
                'activity_date__year': self.object.activity_date.year,
            })
        except DayEntry.DoesNotExist:
            pass
        else:
            return obj
        return None
        
    def get_object(self, **kwargs):
        today = datetime.date.today()
        obj = get_object_or_404(DayEntry, datebook=self.datebook, activity_date=datetime.date(self.year, self.month, self.day))
        obj.projected = False
        if today <= obj.activity_date:
            obj.projected = True
        return obj
    
    def get_context_data(self, **kwargs):
        context = super(DayEntryDetailView, self).get_context_data(**kwargs)
        context.update({
            'datebook': self.datebook,
            'object': self.object,
            'previous_day': self.previous_day,
            'next_day': self.next_day,
            'DATEBOOK_TEXT_MARKUP_RENDER_TEMPLATE': settings.DATEBOOK_TEXT_MARKUP_RENDER_TEMPLATE,
        })
        return context
    
    def get(self, request, *args, **kwargs):
        self.datebook = self.get_datebook({'period__year': self.year, 'period__month': self.month})
        self.object = self.get_object()
        self.previous_day = self.get_previous_day()
        self.next_day = self.get_next_day()
        
        context = self.get_context_data(**kwargs)
        
        return self.render_to_response(context)
    
class DayEntryDeleteFormView(DayEntryBaseFormView, generic.DeleteView):
    template_name = "datebook/day/delete.html"
    permission_required = 'datebook.delete_dayentry'
    
    def get_context_data(self, **kwargs):
        context = super(DayEntryBaseFormView, self).get_context_data(**kwargs)
        context.update({
            'datebook': self.datebook,
            'DATEBOOK_TEXT_FIELD_JS_TEMPLATE': settings.DATEBOOK_TEXT_FIELD_JS_TEMPLATE,
            'DATEBOOK_TEXT_MARKUP_RENDER_TEMPLATE': settings.DATEBOOK_TEXT_MARKUP_RENDER_TEMPLATE,
        })
        return context
    
    def get_object(self):
        return self.datebook.dayentry_set.get(activity_date__day=self.day)

    def get_form(self, form_class):
        return super(DayEntryBaseFormView, self).get_form(form_class)
    
    def get_form_kwargs(self):
        return super(DayEntryBaseFormView, self).get_form_kwargs()

    def get_success_url(self):
        """
        If 'submit_and_next' button has been used, redirect to the next day form, 
        else redirect to the month view
        """
        return reverse('datebook:month-detail', kwargs={
            'author': self.author,
            'year': self.object.activity_date.year,
            'month': self.object.activity_date.month,
        })
        



# -*- coding: utf-8 -*-
"""
Application Crumbs
"""
from autobreadcrumbs import site
from django.utils.translation import ugettext_lazy

site.update({
    'datebook:index': ugettext_lazy('Datebooks'),
    'datebook:create': ugettext_lazy('Create a datebook'),
    'datebook:author-detail': '{{ author.username }}',
    'datebook:year-detail': '{{ year }}',
    'datebook:month-detail': '{{ target_date|date:"F Y" }}',
    'datebook:month-notes': ugettext_lazy('Notes for {{ target_date|date:"F Y" }}'),
    'datebook:day-add': ugettext_lazy('Add {{ target_date|date:"l d F Y" }}'),
    'datebook:day-edit': ugettext_lazy('Edit {{ target_date|date:"l d F Y" }}'),
    'datebook:day-remove': ugettext_lazy('Remove {{ target_date|date:"l d F Y" }}'),
    'datebook:day-models': ugettext_lazy('Day models'),
    'datebook:day-model-edit': ugettext_lazy('Edit "{{ daymodel.title }}"'),
    'datebook:dayentry-to-daymodel': ugettext_lazy('Make a model from {{ target_date|date:"l d F Y" }}'),
})
